/* *******************************************************************

  Weather Monitor
  using Openweathermap

  Hardware
  This sketch was tested with several hardware variants:
  - NodeMCU (or generic ESP8266)
  - ESP32 (open)

  Software
  The Software was last tested with
  - IDE 1.8.119
  - ESP32 Core 1.0.6
  - ESP8266 Core 3.0.2 

  Short
  - Openweather
  - OTA

  Open Tasks
  -

  by noiasca - http://werner.rothschopf.net/microcontroller/202409_esp_weather_en.htm

  Version
  2024-09-07 by noiasca
***************************************************************** */

#define VERSION "24.09"                // the version of this sketch

/* *******************************************************************
         other settings / weitere Einstellungen für den Anwender
 ********************************************************************/

//#include <credentials.h>               // if you have an external file with your credentials you can use it

#ifndef STASSID                        // either use an external .h file containing STASSID and STAPSK - or
//                                     // add defines to your boards -
#define STASSID "your-ssid"            // ... modify these line to your SSID
#define STAPSK  "your-password"        // ... and set your WIFI password
// ... your API key
#endif

const char ssid[] = STASSID;
const char password[] = STAPSK;

#define USE_BOARD 1                    // the actual board to compile
#define TXT_BOARDNAME "Weather"        // the name of the board on webgui and Serial

const uint8_t ouputBPin = 2;           // GPIO02/D4 on NodeMCU is the (blue) LED on the ESP-12E
const uint8_t wifiActivityPin = ouputBPin; // this board has a dedicated LED on the ESP

// Configuration for open weather API 3.0
constexpr uint16_t owInterval {15};              // interval of requests in minutes (60 minutes is a good value when the code is ready)
constexpr char owKey[] {"001122aabbccddeeff"};   // register, subscribe for a plan and get an API key
constexpr float owLat {48.2083};                 // your latitude  (North-South) example for Vienna/Austria
constexpr float owLon {16.3731};                 // your longitude (West-East)
constexpr char owLang[] {"en"};                  // language in Alfa2: en, de
constexpr char owUnits[] {"metric"};             // standard, metric, imperial
constexpr char owExclude[] {"minutely,hourly"};  // exclude minutely and hourly. If you don't need the next 7 days also exclude "daily"

/* *******************************************************************
 The subset of available weather data
 ********************************************************************/
// this data structure will hold all weather information
struct Weather {
  long current_dt;                     // unix timestamp of this data set
  long current_sunrise;                // unix timestamp 1725596374
  long current_sunset;                 // unix timestamp 1725643585
  float current_temp;                  // temperature          26.61
  float current_feels_like;            // temperature (feeled) 26.61
  int current_pressure;                // air presssure       1013
  int current_humidity;                // 42
  float current_dew_point;             // 12.65
  float current_uvi;                   // 0.4
  int current_clouds;                  // 0
  int current_visibility;              // 10000
  float current_wind_speed;            // 5.14
  int current_wind_deg;                // 140
  int current_weather_0_id;            // 800
  String current_weather_0_main;       // "Clear"
  String current_weather_0_description;          // "clear sky"
  String current_weather_0_icon;       // "01d"
  String daily_0_summary;              // "There will be rain until morning, then partly ...

  // conditional alerts
  String alerts_0_sender_name;         // "GeoSphere Austria"
  String alerts_0_event;               // "Heatwarning"
  String alerts_0_description;         // "Increased heat stress must be expected."
  String alerts_0_tags_0;              // "Extreme high temperature"
  
  // conditional error messages
  int cod;                             // 401
  String message;                      // "Please note that using One Call 3.0 requires a..."
} weather;

/* *******************************************************************
         The major libraries needed for this sketch
 ********************************************************************/

#ifdef ARDUINO_ARCH_ESP8266            // libraries if you compile for a ESP8266
#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>               // Bonjour/multicast DNS, finds the device on network by name
#include <ArduinoOTA.h>                // OTA Upload via ArduinoIDE
#include <ESP8266HTTPClient.h>         // Needed for the HTTP client (to get data from an external server)
#endif
#ifdef ARDUINO_ARCH_ESP32              // libraries if you compile for a ESP32
#include <WiFi.h>
#include <ESPmDNS.h>                   // Bonjour/multicast DNS, finds the device on network by name
#include <ArduinoOTA.h>                // OTA Upload via ArduinoIDE
#include <HTTPClient.h>                // Needed for the HTTP client (to get data from an external server)
#endif
#include <ArduinoJson.h>               // from Library Manager Version 7.1.0 (!)

/* *******************************************************************
         necessary includes for this example
 ********************************************************************/

#include "output.h"
#include "openweather.h"               // logic to get data from openweathermap.org
#include "ota.h"                       // Over The Air update

/* *******************************************************************
         Arduino setup() and loop()
 ********************************************************************/
void setup() {
  Serial.begin(115200);
  delay(250); // only for some ESP boards
  Serial.println(F("\nESP " TXT_BOARDNAME "\nVersion: " VERSION ));
  Serial.print  (__DATE__);
  Serial.print  (' ');
  Serial.println(__TIME__);

  Serial.print(F("try "));
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  // Wait for connection
  int8_t i = 0;
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print('.');
    if (wifiActivityPin < 255) digitalWrite(wifiActivityPin, !digitalRead(wifiActivityPin)); // blink one pin as indicator
    i++;
    if (i >= 80) {
      Serial.println();
      i = 0;
    }
    delay(500);
  }
  Serial.print(F("\nConnected to "));
  Serial.println(ssid);
  Serial.print(F("IP address: "));
  Serial.println(WiFi.localIP());
  char myhostname[32] {"esp"};         // you could use your TXT_BOARDNAME also. The sketch will add the board ID.
  char val[8];
  itoa(USE_BOARD, val, 10);            // before we can concatenate the precompiler integer, we have to convert it to char array
  strcat(myhostname, val);
  strcat(myhostname, "weather");
  WiFi.hostname(myhostname);
  if (MDNS.begin(myhostname)) {
    Serial.println(F("MDNS responder started"));
  }
  ArduinoOTASetup();
}

void loop() {
  ArduinoOTA.handle();                 // OTA Upload via ArduinoIDE
  int result = weatherTimer();         // timer to update weather data
  if (result == NEW_DATA) {
    serialOutput();                    // we have new data - show it on Serial, your displays, etc
  }
  outputTimer();                       // timer to ouput the weather data regulary
}
//